DROP TABLE StudentCourse;
DROP TABLE Students;
DROP TABLE Courses;

/*For the purpose of demonstration how join works*/
/*we won't include the foreign key constraint*/
/*since it prevents tables from having unmatch foreigh keys*/
CREATE TABLE Students (
    studentid varchar(10) not null,
    firstname varchar(20) not null,
    lastname varchar(20) not null,
    address varchar(100) not null,
    CONSTRAINT studentpk PRIMARY KEY (studentid)
);

CREATE TABLE Courses (
    courseid varchar(6) not null,
    coursename varchar(50) not null,
    department varchar(50) not null,
    CONSTRAINT coursepk PRIMARY KEY (courseid)
);

CREATE TABLE StudentCourse (
    studentid varchar(10) not null,
    courseid varchar(6) not null,
    CONSTRAINT scpk PRIMARY KEY (studentid, courseid)
--    CONSTRAINT scfk1 FOREIGN KEY (studentid) REFERENCES Students (studentid),
--    CONSTRAINT scfk2 FOREIGN KEY (courseid) REFERENCES Courses (courseid)
);

INSERT INTO Students (studentid, firstname, lastname, address)
          select '0000000001', 'Alice', 'A', 'GA' from dual
union all select '0000000002', 'Bob', 'B', 'GA' from dual
union all select '0000000003', 'Chad', 'C', 'FL' from dual
union all select '0000000004', 'Dan', 'D', 'AL' from dual
union all select '0000000005', 'Emma', 'E', 'FL' from dual
union all select '0000000006', 'Gale', 'G', 'TX' from dual;

INSERT INTO Courses (courseid, coursename, department)
          select 'IT1001', 'Programming Intro', 'Information Technolgy' from dual
union all select 'IT2001', 'Programming Advance', 'Information Technolgy' from dual
union all select 'MA1001', 'Pre-Calculus', 'Math and Science' from dual
union all select 'CS3001', 'Operating System', 'Computer Science' from dual
union all select 'IT4999', 'Thesis', 'Information Technology' from dual;

INSERT INTO StudentCourse (studentid, courseid)
          select '0000000001', 'IT1001' from dual
union all select '0000000002', 'IT2001' from dual
union all select '0000000003', 'MA1001' from dual
union all select '0000000004', 'MA1001' from dual
union all select '0000000005', 'CS3001' from dual
union all select '0000000007', 'CS3311' from dual;

/*sample queries*/
SELECT * FROM Students;
SELECT * FROM Courses;
SELECT * FROM StudentCourse;

/*inner join*/
SELECT s.studentid, s.firstname, s.lastname, sc.courseid
FROM Students s
INNER JOIN StudentCourse sc
ON s.studentid = sc.studentid;

/*left outer join*/
SELECT s.studentid, s.firstname, s.lastname, sc.courseid
FROM Students s
LEFT OUTER JOIN StudentCourse sc
ON s.studentid = sc.studentid;

/*right outer join*/
SELECT s.studentid, s.firstname, s.lastname, sc.courseid
FROM Students s
RIGHT OUTER JOIN StudentCourse sc
ON s.studentid = sc.studentid;

/*full outer join*/
SELECT s.studentid, s.firstname, s.lastname, sc.courseid
FROM Students s
FULL OUTER JOIN StudentCourse sc
ON s.studentid = sc.studentid;

/*join multiple tables*/
SELECT s.studentid, s.firstname, s.lastname, c.courseid, c.coursename
FROM Students s
INNER JOIN StudentCourse sc ON s.studentid = sc.studentid
INNER JOIN Courses c ON sc.courseid = c.courseid;

SELECT s.studentid, s.firstname, s.lastname, c.courseid, c.coursename
FROM Students s
LEFT JOIN StudentCourse sc ON s.studentid = sc.studentid
LEFT JOIN Courses c ON sc.courseid = c.courseid;

SELECT s.studentid, s.firstname, s.lastname, c.courseid, c.coursename
FROM Students s
LEFT JOIN StudentCourse sc ON s.studentid = sc.studentid
RIGHT JOIN Courses c ON sc.courseid = c.courseid;

SELECT s.studentid, s.firstname, s.lastname, c.courseid, c.coursename
FROM Students s
FULL JOIN StudentCourse sc ON s.studentid = sc.studentid
FULL JOIN Courses c ON sc.courseid = c.courseid;


/***UNION***/
SELECT studentid as "id", firstname as "name" 
FROM students
UNION 
SELECT courseid, coursename 
FROM courses;

/*this will remove duplications*/
SELECT studentid 
FROM Students
UNION
SELECT studentid
FROM StudentCourse;

/*this will NOT remove duplications*/
SELECT studentid 
FROM Students
UNION ALL
SELECT studentid
FROM StudentCourse;

/*create faculty table*/
DROP TABLE Faculties;

CREATE TABLE Faculties (
    facultyid VARCHAR(10) NOT NULL,
    firstname VARCHAR(20) NOT NULL,
    lastname VARCHAR(20) NOT NULL,
    department VARCHAR(50) NOT NULL,
    address VARCHAR(100) NOT NULL,
    CONSTRAINT facultypk PRIMARY KEY (facultyid)
);

INSERT INTO Faculties (facultyid, firstname, lastname, department, address)
          select '0000000001', 'Ann', 'A', 'Information Technology','GA' from dual
union all select '0000000002', 'Becky', 'B', 'Information Technology', 'GA' from dual
union all select '0000000003', 'Mike', 'M', 'Computer Science', 'FL' from dual
union all select '0000000004', 'John', 'J', 'Math and Science', 'AL' from dual;

/*Union cont.*/
/*note: */
/**Single-quotes are used to enclose string literals 
(and, in recent versions, DATE literals).
/**Double-quotes are used to enclose identifiers (like table and column names). 
They are optional (and therefore almost never used) 
when the name conforms to certain rules for names 
(starts with a letter, no spaces or special symbols, no lower-case letters, ...).
*/

SELECT studentid as UserID, firstname, lastname, address, 'Student' as "User Type"
FROM Students
UNION
SELECT facultyid, firstname, lastname, address, 'Faculty'
FROM Faculties;

SELECT studentid as "User ID", firstname, lastname, address, 'Student' as "User Type"
FROM Students
UNION
SELECT facultyid, firstname, lastname, address, 'Faculty'
FROM Faculties
ORDER BY "User ID";

SELECT studentid as "User ID", firstname, lastname, address, 'Student' as "User Type"
FROM Students
WHERE address IN ('GA', 'FL')
UNION
SELECT facultyid, firstname, lastname, address, 'Faculty'
FROM Faculties 
WHERE address IN ('GA', 'FL')
ORDER BY address;



/***Views***/
/*with JOIN*/
CREATE OR REPLACE VIEW students_courses AS 
    SELECT s.studentid, s.firstname, s.lastname, c.courseid, c.coursename, c.department
    FROM Students s
    INNER JOIN StudentCourse sc
    ON s.studentid = sc.studentid
    INNER JOIN Courses c
    ON sc.courseid = c.courseid;
    
SELECT * FROM students_courses;

/*with UNION*/
CREATE OR REPLACE VIEW Users AS
    SELECT studentid as UserID, firstname, lastname, address, 'Student' as "User Type"
    FROM Students
    UNION
    SELECT facultyid, firstname, lastname, address, 'Faculty'
    FROM Faculties;
    
SELECT * FROM Users;

/*insert into view*/
CREATE OR REPLACE VIEW StudentSummaries AS
    SELECT studentid, firstname, lastname
    FROM Students;

/*drop the constraint on address not null so we can insert from the view*/
/*this is just an example, and is a really bad practice*/
/*find the constraint name since it's automatically generated by Oracle*/
SELECT *
FROM user_cons_columns
WHERE LOWER(table_name) = 'students';

ALTER TABLE Students
    DROP CONSTRAINT SYS_C0010017;
    
INSERT INTO StudentSummaries (studentid, firstname, lastname)
VALUES ('000000010','Mark','M');

SELECT * FROM Students;