/*dropping tables in case they are already there*/
DROP TABLE Orders;
DROP TABLE Customers;

/*create table customers, set customer_id to be primary key*/
CREATE TABLE Customers (
    customer_id varchar(10) NOT NULL,
    first_name varchar(20) NOT NULL,
    last_name varchar(20) NOT NULL,
    phone varchar(12) NULL,
    CONSTRAINT customer_pk PRIMARY KEY (customer_id)
);

/*create table orders, set order_id to be primary key, set customer_id to be foreign key*/
CREATE TABLE Orders (
    order_id varchar(10) NOT NULL,
    datetime timestamp(6) NOT NULL,
    total number(10,2) NOT NULL,
    customer_id NOT NULL,
    CONSTRAINT order_pk PRIMARY KEY (order_id),
    CONSTRAINT order_fk1 FOREIGN KEY (customer_id)
        REFERENCES Customers (customer_id)
);

/*add two columns to table customers*/
ALTER TABLE Customers 
    ADD (
        age number(2) NULL,
        salary number(10)
        )
;

/*populate fake data into table customers*/
/*this is one way to insert multiple rows into a table in oracle*/
INSERT INTO Customers (customer_id, first_name, last_name, phone, age, salary)
          select '0000000001', 'Alice', 'A', '1234567890', 30, 50000 from dual
union all select '0000000002', 'Bob', 'B', '12345678901', 35, 60000 from dual
union all select '0000000003', 'Chad', 'C', '2234567890', 30, 55000 from dual
union all select '0000000004', 'Dan', 'D', '1235567890', 25, 40000 from dual
union all select '0000000005', 'Emma', 'E', '1234568790', 30, 65000 from dual
union all select '0000000006', 'Fran', 'F', '4334567890', 35, 70000 from dual
union all select '0000000007', 'Gale', 'G', '1234569990', 25, 60000 from dual
union all select '0000000008', 'Han', 'H', '3334569990', 25, 80000 from dual
union all select '0000000009', 'Ivan', 'I', '1234569802', 40, 80000 from dual
union all select '0000000010', 'Jake', 'J', '1344569990', 40, 70000 from dual;

/*query the whole customers table*/
SELECT * FROM Customers;

/*query three specific columns in table customers*/
SELECT first_name, last_name, salary 
FROM  Customers;

/*query three columns in table customers, filter by age above 30*/
SELECT first_name, last_name, salary
FROM Customers
WHERE age > 30;

/*count the number of records in table customers*/
SELECT COUNT(*) FROM Customers;

/*count the number of unique values of age in table customers*/
SELECT COUNT(UNIQUE age) FROM Customers;

/*get the average salary and age of all customers*/
SELECT AVG(salary), AVG(age)
FROM Customers;

/*get the average salary of all customers older than 30*/
SELECT AVG(salary)
FROM Customers
WHERE age > 30;

/*get the average salary of customers grouped by their ages*/
SELECT age, AVG(salary)
FROM Customers
GROUP BY age;

/*get the average salary of customers grouped by 
their ages then sort by age ascending*/
SELECT age, AVG(salary)
FROM Customers
GROUP BY age
ORDER BY age ASC;

/*get the average salary of customers with salary 
higher than 60000 grouped by their ages*/
SELECT age, AVG(salary)
FROM Customers
WHERE salary > 60000
GROUP BY age;

/*get the average salary of customers grouped by their ages, 
then filtered out rows with average salary less than 60000*/
SELECT age, AVG(salary)
FROM Customers
GROUP BY age
HAVING AVG(salary) > 60000;

