DROP TABLE Employees;

CREATE TABLE Employees (
    emp_id VARCHAR(5) NOT NULL,
    first_name VARCHAR(5) NOT NULL,
    last_name VARCHAR(5) NOT NULL,
    income NUMBER NOT NULL,
    experience NUMBER NOT NULL,
    CONSTRAINT emppk PRIMARY KEY (emp_id)
);

INSERT INTO Employees (emp_id, first_name, last_name, income, experience)
          select '00001', 'Alice', 'A', 60000, 1 from dual
union all select '00002', 'Bob', 'B', 65000, 2 from dual
union all select '00003', 'Chad', 'C', 55000, 1 from dual
union all select '00004', 'Dan', 'D', 75000, 3 from dual
union all select '00005', 'Emily', 'E', 80000, 4 from dual
;

/*IF THEN ELSE*/
SET SERVEROUTPUT ON;

DECLARE
    x NUMBER := 10;
BEGIN
    IF x < 20 THEN
        dbms_output.put_line('x is less than 20');
    ELSE 
        dbms_output.put_line('x is greater than 20');
    END IF;
END;

/**/
CREATE OR REPLACE PROCEDURE raise_salary(eid VARCHAR)
AS
    e_exp NUMBER;
BEGIN
    SELECT experience INTO e_exp
    FROM Employees
    WHERE emp_id = eid;
    
    IF e_exp <= 2 THEN
        UPDATE Employees SET income = income * 1.05 WHERE emp_id = eid;
    ELSIF e_exp < 2 AND e_exp <= 3 THEN
        UPDATE Employees SET income = income * 1.10 WHERE emp_id = eid;
    ELSE
        UPDATE Employees SET income = income * 1.15 WHERE emp_id = eid;
    END IF;
END;

EXECUTE raise_salary('00001');
EXECUTE raise_salary('00003');
EXECUTE raise_salary('00005');

SELECT * FROM Employees;


/*CASE*/
SELECT emp_id, first_name, last_name,
       CASE experience
       WHEN 1 THEN 'New Employee'
       WHEN 2 THEN 'Junior Employee'
       WHEN 3 THEN 'Junior Employee'
       WHEN 4 THEN 'Senior Employee'
       END
       AS emp_level
FROM Employees;

/*2nd way*/
SELECT emp_id, first_name, last_name,
       CASE
       WHEN experience = 1 THEN 'New Employee'
       WHEN experience IN (2,3) THEN 'Junior Employee'
       WHEN experience = 4 THEN 'Senior Employee'
       END
       AS emp_level
FROM Employees;

/**/
CREATE OR REPLACE FUNCTION income_level(inc NUMBER) RETURN VARCHAR
AS
BEGIN
    CASE
    WHEN inc < 60000 THEN RETURN 'Low Income';
    WHEN (inc >= 60000 AND inc < 75000) THEN RETURN 'Medium Income';
    WHEN inc >= 75000 THEN RETURN 'High Income';
    END CASE;
END;

SELECT emp_id, first_name, last_name, income_level(income)
FROM Employees;