SET SERVEROUTPUT ON;

DECLARE
    s VARCHAR(20) := 'Hello World!';
    x NUMBER := 1.5;
    y CONSTANT INTEGER := 5;
    z CONSTANT REAL := 10.25;
    flag BOOLEAN := TRUE;
    today DATE := '04-AUG-19';
    next_day DATE;
BEGIN
    dbms_output.put_line(s);
    dbms_output.put_line('2x = ' || x * 2);
    dbms_output.put_line(today);
    next_day := today + 1;
    dbms_output.put_line(next_day);
    dbms_output.put_line(y || ' and ' || z);
    /*modifying a constant will yield error*/
    /*y := y + 1;*/
END;

/*change schema to samplescdb to access OE tables*/
ALTER SESSION SET CONTAINER=samplescdb;

SELECT owner, table_name FROM dba_tables WHERE owner='OE';

SELECT * from oe.customers where ROWNUM <=10;

/**Declaring variables using a column type and assigning values from the table**/
SET SERVEROUTPUT ON;
DECLARE
    cus_id oe.customers.customer_id%TYPE;
    donald_num NUMBER;
BEGIN
    SELECT customer_id INTO cus_id
    FROM oe.customers
    WHERE cust_first_name = 'Donald' AND ROWNUM < 2;
    
    SELECT COUNT(*) INTO donald_num
    FROM oe.customers
    WHERE cust_first_name = 'Donald';
    
    dbms_output.put_line('First Donald IDs: ' || cus_id);
    dbms_output.put_line('Number of Donald: ' || donald_num);
END;

/*User-defined type*/
DECLARE
    TYPE Box IS RECORD(
        width NUMBER,
        height NUMBER,
        "length" NUMBER
    );
    box1 Box;
BEGIN
    box1.width := 10;
    box1.height := 12;
    box1."length" := 15;
    dbms_output.put_line('Box dimension: ' || box1.width || 'x' 
                         || box1.height || 'x' 
                         || box1."length");
END;

DECLARE
    TYPE CustomerInfo IS RECORD(
        cid oe.customers.customer_id%TYPE,
        firstname oe.customers.cust_first_name%TYPE,
        lastname oe.customers.cust_last_name%TYPE        
    );
    a_customer CustomerInfo;
BEGIN
    SELECT customer_id, cust_first_name, cust_last_name INTO a_customer
    FROM oe.customers
    WHERE ROWNUM < 2;
    
    dbms_output.put_line('A customer: ' || a_customer.cid || ' - ' 
                         || a_customer.firstname || ' - ' 
                         || a_customer.lastname);
END;

/*define type of variables as a row of a table*/
DECLARE
    customer_1 oe.customers%ROWTYPE;
BEGIN
    SELECT * INTO customer_1 
    FROM oe.customers
    WHERE rownum < 2;
    
    dbms_output.put_line(customer_1.customer_id || ' - ' ||
                         customer_1.cust_first_name || ' - ' ||
                         customer_1.cust_last_name || ' - ' ||
                         customer_1.cust_email);
END;


/**Exception Handling**/
/*the code below will cause error*/
DECLARE
    x NUMBER := 0;
BEGIN
    x := 5/x;
    dbms_output.put_line('x = ' || x);
END;

/*we can add exception and handle it in our way instead*/
DECLARE
    x NUMBER := 0;
BEGIN
    x := 5/x;
EXCEPTION
    WHEN ZERO_DIVIDE THEN
        dbms_output.put_line('Dividing by 0, value of x unchanged');
        dbms_output.put_line('x = ' || x);
END;


DECLARE
    customer_1 oe.customers%ROWTYPE;
BEGIN
    SELECT * INTO customer_1 
    FROM oe.customers
    WHERE rownum < 2;
    
    dbms_output.put_line(customer_1.customer_id || ' - ' ||
                         customer_1.cust_first_name || ' - ' ||
                         customer_1.cust_last_name || ' - ' ||
                         customer_1.cust_email);
                         
    SELECT * INTO customer_1 
    FROM oe.customers
    WHERE rownum < 1;
    
EXCEPTION
    WHEN NO_DATA_FOUND THEN
        dbms_output.put_line('no records found, set all fields to null');
        customer_1 := NULL;
        dbms_output.put_line(customer_1.customer_id || ' - ' ||
                             customer_1.cust_first_name || ' - ' ||
                             customer_1.cust_last_name || ' - ' ||
                             customer_1.cust_email);
END;
/***/

/**Oracle built-in functions**/
SELECT length('asdfgh') from DUAL;	 --the length of the string asdfgh
SELECT user from DUAL; 	--user who runs this code
SELECT (5+6) from dual; 	--the result of addition of 5 and 6
SELECT round(4.567,1) from dual; 	--4.567 rounded to the nearest tenth 
SELECT dbms_random.random() from dual; 	--generate a random number
SELECT sysdate from DUAL;	--current date
SELECT to_char (sysdate,'DD MONTH YYYY') from dual; --current date in a different format 

/**Stored procedure**/
CREATE OR REPLACE PROCEDURE print_string 
AS 
    str VARCHAR(20) := 'Hello Worlds';
BEGIN
	dbms_output.put_line(str);
END;

EXECUTE print_string;

/**/
CREATE OR REPLACE PROCEDURE count_customers
AS
    cust_num NUMBER;
BEGIN
    SELECT COUNT(*) INTO cust_num
    FROM oe.customers;
    dbms_output.put_line('There are ' || cust_num || ' customers in the table');
END;

EXECUTE count_customers;

/**procedures with input parameters**/
CREATE OR REPLACE PROCEDURE find_customers_by_name(cname VARCHAR2)
AS
    TYPE CustomerInfo IS RECORD(
        cid oe.customers.customer_id%TYPE,
        firstname oe.customers.cust_first_name%TYPE,
        lastname oe.customers.cust_last_name%TYPE,
        email oe.customers.cust_email%TYPE
    );
    customer CustomerInfo;
BEGIN
    SELECT customer_id, cust_first_name, cust_last_name, cust_email INTO customer
    FROM oe.customers
    WHERE (LOWER(cust_first_name) LIKE '%'||cname||'%' OR 
           LOWER(cust_last_name) LIKE '%'||cname||'%') 
           AND rownum < 2;
          
    dbms_output.put_line(customer.cid || ' - ' ||
                         customer.firstname || ' - ' ||
                         customer.lastname || ' - ' ||
                         customer.email);      
END;

EXECUTE find_customers_by_name('donald');
EXECUTE find_customers_by_name('alice');
EXECUTE find_customers_by_name('ha ha ha');

/**/
CREATE OR REPLACE PROCEDURE display_customer_by_id(cid NUMBER)
AS
    TYPE CustomerInfo IS RECORD(
        cid oe.customers.customer_id%TYPE,
        firstname oe.customers.cust_first_name%TYPE,
        lastname oe.customers.cust_last_name%TYPE,
        email oe.customers.cust_email%TYPE
    );
    customer CustomerInfo;
    
BEGIN
    SELECT customer_id, cust_first_name, cust_last_name, cust_email INTO customer
    FROM oe.customers
    WHERE customer_id = cid;
    
    dbms_output.put_line(customer.cid || ' - ' ||
                         customer.firstname || ' - ' ||
                         customer.lastname || ' - ' ||
                         customer.email);
    
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
        dbms_output.put_line('no records found');
END;

EXECUTE display_customer_by_id(232);
EXECUTE display_customer_by_id(254);
EXECUTE display_customer_by_id(1231);



/***Functions***/
CREATE OR REPLACE FUNCTION sum_numbers(x NUMBER, y NUMBER) RETURN NUMBER
AS
BEGIN
    RETURN x + y;
END;

SELECT sum_numbers(10,23) FROM dual;


CREATE OR REPLACE FUNCTION concat_and_upper(s1 VARCHAR, s2 VARCHAR) RETURN VARCHAR
AS
BEGIN
    RETURN UPPER(s1) || ' ' || UPPER(s2);
END;

SELECT customer_id, cust_first_name, cust_last_name,
       concat_and_upper(cust_first_name, cust_last_name) as "Full Name"
FROM oe.customers
WHERE rownum < 10;