DROP TABLE products;
DROP TABLE product_logs;

CREATE TABLE products (
    pid INT GENERATED BY DEFAULT ON NULL AS IDENTITY,
    pname VARCHAR(50) NOT NULL,
    price NUMBER CHECK (price >= 0),
    CONSTRAINT productpk PRIMARY KEY (pid)
);

/*create log table*/
CREATE TABLE product_logs(
    log_id INT GENERATED BY DEFAULT ON NULL AS IDENTITY,
    pid INT NOT NULL,
    action VARCHAR(200) NOT NULL,
    from_user VARCHAR(50) NOT NULL,
    datetime TIMESTAMP DEFAULT SYSDATE,
    CONSTRAINT log_pk PRIMARY KEY (log_id)
);

CREATE OR REPLACE TRIGGER log_insert_products
BEFORE INSERT ON products
FOR EACH ROW
BEGIN
    INSERT INTO product_logs (pid, action, from_user)
    VALUES (:new.pid, 'insert new', user);
END;



INSERT INTO products (pname, price) VALUES ('Pen',2.5);
INSERT INTO products (pname, price) VALUES ('Pencil',3.5);
INSERT INTO products (pname, price) VALUES ('Sharpie',4);
INSERT INTO products (pname, price) VALUES ('Notebook',2);

SELECT * FROM products;
SELECT * FROM product_logs;

/*assuming we want to log the old value and new value also*/
/*we will create a new variable to carry the message*/
/*this is for demonstration purpose*/
/*for normalized database, you may want to create a separate actions table*/
CREATE OR REPLACE TRIGGER log_update_products
BEFORE UPDATE ON products
FOR EACH ROW
DECLARE
    actions VARCHAR2(200);
BEGIN
    IF UPDATING('pname') THEN actions := actions || 
        'update name from ' || :old.pname || ' to ' || :new.pname || '; ';
    END IF;
    IF UPDATING('price') THEN actions := actions || 
        'update price from ' || :old.price || ' to ' || :new.price || '; ';
    END IF;
    INSERT INTO product_logs (pid, action, from_user)
    VALUES (:new.pid, actions, user);
END;

UPDATE products SET pname = 'Fountain Pen' WHERE pid = 1;
UPDATE products SET pname = '300-page Notebook', price=3.5 WHERE pid = 4;

SELECT * FROM product_logs;


/*trigger to log both insert and update*/
/*we need to disable the previous two triggers so they don't overlap*/
ALTER TRIGGER log_insert_products DISABLE;
ALTER TRIGGER log_update_products DISABLE;

/*here we use case/when to detect the action (insert or update)*/
/*and if/then to detect the columns that are being updated*/
CREATE OR REPLACE TRIGGER log_update_products
BEFORE UPDATE OR INSERT ON products
FOR EACH ROW
DECLARE
    actions VARCHAR2(200);
BEGIN
    CASE 
        WHEN INSERTING THEN actions := 'insert new row';
        WHEN UPDATING THEN 
            IF UPDATING('pname') THEN actions := actions || 
                'update name from ' || :old.pname || ' to ' || :new.pname || '; ';
            END IF;
            IF UPDATING('price') THEN actions := actions || 
                'update price from ' || :old.price || ' to ' || :new.price || '; ';
            END IF;
        END CASE;
    INSERT INTO product_logs (pid, action, from_user)
    VALUES (:new.pid, actions, user);
END;


INSERT INTO products (pname, price) VALUES ('Marker',4.0);
UPDATE products SET pname = 'Yellow Marker', price=3.5 WHERE pid = 5;

SELECT * FROM product_logs;

/**how to add log for delete to the trigger?**/