/*create a test table with some indexes*/
DROP TABLE testtable;

CREATE TABLE testtable(
    x1 INT PRIMARY KEY,
    x2 INT UNIQUE,
    x3 INT,
    x4 VARCHAR(5) UNIQUE USING INDEX (
        CREATE INDEX x4_indx ON testtable(x4)),
    x5 long    
);

/*check all constraint from testtable*/
SELECT * FROM all_indexes
WHERE table_name = 'TESTTABLE';
/*we can get more details of the indexes using dba_ind_columns*/
/*as you see, primary key x1 and x2 with unique constraint are*/
/*automatically generated their indexes*/
/*and x4 has the explicitly created index*/
SELECT * FROM DBA_IND_COLUMNS 
WHERE table_name = 'TESTTABLE';

/*create more index and check again*/
CREATE INDEX x3_indx ON testtable (x3);
SELECT * FROM DBA_IND_COLUMNS 
WHERE table_name = 'TESTTABLE';

/*long column cannot be indexed*/
CREATE INDEX x5_indx ON testtable (x5);

/*drop an index and check again*/
DROP INDEX x3_indx;
SELECT * FROM DBA_IND_COLUMNS 
WHERE table_name = 'TESTTABLE';

/*and we cannot drop an index for the unique/primary key constraint*/
DROP INDEX x4_indx;

/*we need to drop the constraint instead first*/
/*first we need to determine the constraint name*/
SELECT * FROM user_cons_columns
WHERE table_name = 'TESTTABLE';
/*mine query returns the constraint for x4 is named "SYS_C0017868"*/
/*now we drop the constraint*/
ALTER TABLE testtable
DROP CONSTRAINT SYS_C0017868;
/*finally we can drop the index*/
DROP INDEX x4_indx;
/*and check the indexes of the table again*/
SELECT * FROM DBA_IND_COLUMNS 
WHERE table_name = 'TESTTABLE';